# coding=utf-8


"""
rdee-series toolkits: python-easyarg

Used to call target function in CLI easily and compatibly, i.e., it requires nothing to do with source code except for a decorator
"""

from .easyarg import EasyArg
